###############################################################################
# zapplet - A tray applet for interfacing with Zenoss
# Copyright (C) 2008 Nathaniel McCallum <nathaniel@natemccallum.com>
# 
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2.1 of the License, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful, but WITHOUT 
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
# 
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
###############################################################################

import gtk

class StatusIconApplet(gtk.StatusIcon):

	def __on_status_icon_activate(self, si):
		if (self.__window.get_property("visible")):
			self.__window.hide_all();
			return;

		self.__window.show_all();
		
		# Get the screen, position, panel orientation and window size
		screen, position, panel = self.get_geometry()
		ww, wh = self.__window.get_size()
		
		# The panel is horizontal
		if panel == gtk.ORIENTATION_HORIZONTAL:
			# If the icon is on the left side
			if position.x + position.width < screen.get_width() / 2:
				x = (position.x + position.width - ww) > 0 and position.x + position.width - ww or 0
			
			# If the icon is on the right side
			else:
				x = (screen.get_width() - ww) > position.x and position.x or screen.get_width() - ww;
			
			# If the panel is at the bottom of the screen
			if screen.get_height() - position.y < screen.get_height() / 2:
				y = screen.get_height() - position.height - wh - 2;
			
			# If the panel is at the top of the screen
			else:
				y = position.y + position.height + 2;

		# The panel is vertical
		else:
			# When the panel is vertical, always have the window hanging down
			# To do anything else makes the window appear anti-gravity, which looks wierd
			y = (screen.get_height() - wh) > position.y and position.y or screen.get_height() - wh;

			# If the panel is on the right side of the screen
			if screen.get_width() - position.x < screen.get_width() / 2:
				x = screen.get_width() - position.width - ww - 2;

			# If the panel is at the left side of the screen
			else:
				x = position.x + position.width + 2;	

		self.__window.move(x, y)
		self.__window.present()

	def __on_status_icon_popup_menu(self, si, button, activate_time):
		if self.menu.get_property("visible"):
			self.menu.popdown()
		else:	
			self.menu.show_all()
			self.menu.popup(None, None, None, button, activate_time)

	def __init__(self, child):
		super(StatusIconApplet, self).__init__()

		if not isinstance(child, gtk.Widget):
			raise TypeError()

		self.__window = gtk.Window()
		self.__window.add(child)
		self.__window.set_decorated(False)
		self.__window.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_SPLASHSCREEN)
		self.__window.set_border_width(10)
		self.__window.set_keep_above(True)
		
		self.connect("activate", self.__on_status_icon_activate)
		self.connect("popup-menu", self.__on_status_icon_popup_menu)

		self.__menu = gtk.Menu()

	child = property(lambda s: s.__window.child)
	menu  = property(lambda s: s.__menu)
